use serde::{Deserialize, Serialize};
use crate::{DockerTags, QueryArgs};

/// registry api response struct
#[derive(Serialize, Deserialize)]
struct RegistryTagsResponse {
    name: Option<String>,
    tags: Option<Vec<String>>,
}

#[derive(Serialize, Deserialize)]
struct RegistryTokenResponse {
    token: Option<String>,
    access_token: Option<String>,
    expires_in: u32,
    issued_at: Option<String>,
}


pub struct RegistryTagsFetcher {}

impl DockerTags for RegistryTagsFetcher {
    fn get_tags(&self, args: &QueryArgs) -> Vec<String> {
        let mut namespace = "library";
        let repository = match &args.repository {
            Some(s) => {
                let vec = s.split('/')
                    .into_iter()
                    .collect::<Vec<&str>>();
                namespace = vec[0];
                vec[1]
            }
            None => { "" }
        };
        let token = get_token(namespace, repository).unwrap();
        let url = format!("https://registry-1.docker.io/v2/{namespace}/{repository}/tags/list");
        let client = reqwest::blocking::Client::new();
        let response = client.get(url)
            .header("Authorization", format!("Bearer {token}"))
            .send();
        let mut results = Vec::<String>::new();
        if let Ok(r) = response {
            let json: RegistryTagsResponse = serde_json::from_str(r.text().unwrap().as_str()).unwrap();
            results.append(&mut json.tags.unwrap());
        };
        results
    }
}

fn get_token(namespace: &str, repository: &str) -> reqwest::Result<String> {
    let username = "simaek";
    let password = "snow@PX1314";
    let service = "registry.docker.io";
    let scope = format!("repository:{namespace}/{repository}:pull");
    let auth = format!("https://auth.docker.io/token?service={service}&scope={scope}");
    let client = reqwest::blocking::Client::new();
    let response = client.get(auth)
        .basic_auth(username, Some(password))
        .send();
    match response {
        Ok(r) => {
            let json: RegistryTokenResponse = serde_json::from_str(r.text().unwrap().as_str()).unwrap();
            Ok(json.token.unwrap())
        }
        Err(e) => { Err(e) }
    }
}
